#!/usr/bin/env python3

import numpy as np
import matplotlib.pyplot as plt
from 	matplotlib.animation 	import FuncAnimation
# from    matplotlib              import cm
import matplotlib
import pandas as pd
from scipy import integrate

import os
import matplotlib.gridspec as gridspec
from matplotlib import rcParams
from matplotlib.ticker import MultipleLocator

rcParams['axes.labelsize'] =11
rcParams['xtick.labelsize'] =11
rcParams['ytick.labelsize'] =11
rcParams['legend.fontsize'] =11
rcParams['font.sans-serif'] = 'Arial'
rcParams['text.usetex'] = False
rcParams['figure.autolayout'] = True
rcParams['lines.markersize'] = 5
rcParams['figure.figsize'] = (7, 6)
rcParams['axes.grid'] = True
rcParams['grid.color'] = 'lightGrey'
rcParams['axes.axisbelow'] = True

class plotter_v3:
    # This plotter will make 2d plots of up to two results folders

    # paths
    input_paths = []
    parameter = None

    # Figures and axes
    fig_square = False
    fig = None
    gs = None
    ax1 = None
    plot_every= 1

    max_values = []

    # Data
    input_data = None
    limits = None

    time1 = []
    time2 = []

    max_values1 = []
    max_values2 = []

    maxlim = -1e99
    minlim = 1e99
    xlim = None

    def __init__(self, input_folder, xlim = None, parameter=None):
        self.input_paths = input_folder
        self.parameter = parameter
        self.xlim = xlim

        if self.fig_square:
            # Figures and axes
            self.fig = plt.figure(figsize=(12, 3.5))
            self.ax1 = self.fig.add_subplot()
        else:
            # Figures and axes
            # self.fig = plt.figure(figsize=(8, 4))
            self.fig = plt.figure(figsize=(6,5))
            self.fig2 = plt.figure(figsize=(10,5))
            self.fig3 = plt.figure(figsize=(6,5))
            self.ax1 = self.fig.add_subplot(111)
            self.ax2 = self.fig2.add_subplot(111)
            self.ax3 = self.fig3.add_subplot(111)

        # Making the figures:
        # https://matplotlib.org/stable/gallery/subplots_axes_and_figures/broken_axis.html
        # self.fig, (self.ax1, self.ax2) = plt.subplots(1, 2, sharey=True)
        self.fig.subplots_adjust(wspace=0.035)

        # Loading the data

        data = pd.read_csv(self.input_paths + os.sep + 'Power_per_Coil.csv')
        data2 = pd.read_csv(self.input_paths + os.sep + 'Power_per_Solenoid.csv')
        columns = data.columns
        columns2 = data2.columns

        # cmap = cm.get_cmap('viridis')
        cmap = matplotlib.colormaps['coolwarm']
        colors = [cmap(i) for i in np.linspace(0, 1.0, len(columns))]
        x = data['time'].values
        for i in range(len(columns)):
            if columns[i] != 'time':
                y = data[columns[i]].values
                self.ax1.plot(x, y*1e-6, color=colors[i])

        cmap = matplotlib.colormaps['coolwarm']
        colors2 = [cmap(i) for i in np.linspace(0, 1.0, len(columns2))]
        for i in range(len(columns2)):
            if columns2[i] != 'time':
                y = data2[columns2[i]].values
                self.ax2.plot(x, y*1e-6, color=colors2[i], label=columns2[i])
                self.ax3.plot(x, integrate.cumulative_trapezoid(y, x, initial=0), color=colors2[i], label=columns2[i])

        #

        self.ax1.set_xlabel('Time, s')
        self.ax2.set_xlabel('Time, s')
        self.ax1.set_ylabel('Power, MW')
        self.ax2.set_ylabel('Power, MW')
        if self.xlim is not None:
            self.ax1.set_xlim((self.xlim[0], self.xlim[1]))
            self.ax2.set_xlim((self.xlim[0], self.xlim[1]))

        self.ax2.legend()
        self.ax3.legend()

        # self.ax2.set_xlabel('Time, s')
        # self.ax2.set_xlim((-0.1, 0.8))

        # self.ax2.legend(loc='lower right', ncols=2)



        # self.ax1.grid()


        # hide the spines between ax and ax2
        # self.ax1.spines.right.set_visible(False)
        # self.ax2.spines.left.set_visible(False)
        # self.ax1.xaxis.tick_bottom()
        # self.ax1.tick_params(labeltop=False)  # don't put tick labels at the top
        # self.ax2.xaxis.tick_bottom()
        # self.ax2.tick_params(left = False, labelleft = False)

        # Now, let's turn towards the cut-out slanted lines.
        # We create line objects in axes coordinates, in which (0,0), (0,1),
        # (1,0), and (1,1) are the four corners of the axes.
        # The slanted lines themselves are markers at those locations, such that the
        # lines keep their angle and position, independent of the axes size or scale
        # Finally, we need to disable clipping.

        # d = 2.0  # proportion of vertical to horizontal extent of the slanted line
        # kwargs = dict(marker=[(-1, -d), (1, d)], markersize=12,
        #               linestyle="none", color='k', mec='k', mew=1, clip_on=False)
        # self.ax1.plot([1, 1], [0, 1], transform=self.ax1.transAxes, **kwargs)
        # self.ax2.plot([0, 0], [0, 1], transform=self.ax2.transAxes, **kwargs)


        plt.setp(self.ax1.get_xticklabels()[-1], visible=False)

        for axis in ['top', 'bottom', 'left']:
            self.ax1.spines[axis].set_linewidth(1.0)

        self.ax1.xaxis.set_tick_params(direction='in', top='on', width=1.2, length=5)
        self.ax1.yaxis.set_tick_params(direction='in', left='on', width=1.2, length=5)
        self.ax1.xaxis.set_tick_params(direction='in', top='on', which='minor', width=1.0, length=3)
        self.ax1.yaxis.set_tick_params(direction='in', left='on', which='minor', width=1.0, length=3)

        self.ax1.xaxis.set_tick_params(direction='in', top='on', width=1.2, length=5)
        self.ax1.yaxis.set_tick_params(direction='in', right='on', width=1.2, length=5)
        self.ax1.xaxis.set_tick_params(direction='in', top='on', which='minor', width=1.0, length=3)
        self.ax1.yaxis.set_tick_params(direction='in', right='on', which='minor', width=1.0, length=3)

        # self.ax1.xaxis.set_minor_locator(MultipleLocator(0.1))
        # self.ax1.xaxis.set_major_locator(MultipleLocator(0.2))

        # self.ax1.yaxis.set_minor_locator(MultipleLocator(10.0))
        # self.ax1.yaxis.set_major_locator(MultipleLocator(20.0))

        # self.ax2.xaxis.set_tick_params(direction='in', top='on', width=1.2, length=5)
        # self.ax2.yaxis.set_tick_params(direction='in', right='on', width=1.2, length=5)
        # self.ax2.xaxis.set_tick_params(direction='in', top='on', which='minor', width=1.0, length=3)
        # self.ax2.yaxis.set_tick_params(direction='in', right='on', which='minor', width=1.0, length=3)

        # self.ax2.xaxis.set_minor_locator(MultipleLocator(0.1))
        # self.ax2.xaxis.set_major_locator(MultipleLocator(0.2))

        # self.ax2.yaxis.set_minor_locator(MultipleLocator(25.0))
        # self.ax2.yaxis.set_major_locator(MultipleLocator(50.0))

        self.fig.tight_layout()
        self.fig2.tight_layout()
        # Poster
        self.fig.savefig(self.input_paths + os.sep + 'Power_per_element.png', dpi=1000)
        self.fig2.savefig(self.input_paths + os.sep + 'Power_per_coil.png', dpi=1000)



        plt.show()

if __name__ == "__main__":
    # inputs = ['C:/Tim Mulder/CERNBox/Staff - TE-MPE-PE/Projects/ni-coils/geometry/output/test_geometry_28_FCS/results/result - 2024-05-28 - 0.1 - 2',
    #            'C:/Tim Mulder/CERNBox/Staff - TE-MPE-PE/Projects/ni-coils/geometry/output/test_geometry_28_FCS/results/result - 2024-05-28 - 0.1 - 1',
    #           'C:/Tim Mulder/CERNBox/Staff - TE-MPE-PE/Projects/ni-coils/geometry/output/test_geometry_28_FCS/results/result - 2024-05-28 - 0.1 - 3',
    #           'C:/Tim Mulder/CERNBox/Staff - TE-MPE-PE/Projects/ni-coils/geometry/output/test_geometry_28_FCS/results/result - 2024-05-28 - 0.1 - 4',]
    # labels = ['1) 10 ms prot. delay', '2) 100 ms prot. delay', '3) No capacitor', '4) No protection']
    # inputs = ['C:/Tim Mulder/CERNBox/Staff - TE-MPE-PE/Projects/ni-coils/geometry/output/FCS_750/results/result - 2024-07-31 - 0.1 - 22',   # GOOD fast protection
    #             'C:/Tim Mulder/CERNBox/Staff - TE-MPE-PE/Projects/ni-coils/geometry/output/FCS_750/results/result - 2024-07-31 - 0.1 - 19',   # Good 300 V capacitor
    #           'C:/Tim Mulder/CERNBox/Staff - TE-MPE-PE/Projects/ni-coils/geometry/output/FCS_750/results/result - 2024-07-31 - 0.1 - 18',
    #           'C:/Tim Mulder/CERNBox/Staff - TE-MPE-PE/Projects/ni-coils/geometry/output/FCS_750/results/result - 2024-07-31 - 0.1 - 17',
    #           'C:/Tim Mulder/CERNBox/Staff - TE-MPE-PE/Projects/ni-coils/geometry/output/FCS_750/results/result - 2024-07-31 - 0.1 - 20',   # GOOD Only breaker
    #           'C:/Tim Mulder/CERNBox/Staff - TE-MPE-PE/Projects/ni-coils/geometry/output/FCS_750/results/result - 2024-07-31 - 0.1 - 21',   # GOOD No protection

              # 'C:/Tim Mulder/CERNBox/Staff - TE-MPE-PE/Projects/ni-coils/geometry/output/FCS_375/results/result - 2024-08-01 - 0.1 - 2',
              # 'C:/Tim Mulder/CERNBox/Staff - TE-MPE-PE/Projects/ni-coils/geometry/output/FCS_375/results/result - 2024-08-01 - 0.1 - 3',
              # 'C:/Tim Mulder/CERNBox/Staff - TE-MPE-PE/Projects/ni-coils/geometry/output/FCS_375/results/result - 2024-08-01 - 0.1 - 4',
              # 'C:/Tim Mulder/CERNBox/Staff - TE-MPE-PE/Projects/ni-coils/geometry/output/FCS_375/results/result - 2024-08-01 - 0.1 - 5',
              # 'C:/Tim Mulder/CERNBox/Staff - TE-MPE-PE/Projects/ni-coils/geometry/output/FCS_375/results/result - 2024-08-01 - 0.1 - 6',
              # 'C:/Tim Mulder/CERNBox/Staff - TE-MPE-PE/Projects/ni-coils/geometry/output/FCS_375/results/result - 2024-08-01 - 0.1 - 7',
              # 'C:/Tim Mulder/CERNBox/Staff - TE-MPE-PE/Projects/ni-coils/geometry/output/FCS_750/results/result - 2024-08-01 - 0.1 - 1',
              # ]
    # labels = ['CD 300V 50ms', 'CD 900V', 'CD 600V', 'CD 300V', 'Only breaker','No protection','7','8','9','10','11','12','13','14','15','16','17','18']
    # input_folder = 'C:/Tim Mulder/CERNBox/Staff - TE-MPE-PE/Projects/ni-coils/geometry/output/test_geometry_29_AMS_100_v2/results/result - 2024-07-03 - 0.1 - 5'
    input_folder = 'C:/Tim Mulder/CERNBox/Staff - TE-MPE-PE/Projects/ni-coils/geometry/output/test_geometry_29_AMS_100_v3/results/result - 2024-09-23 - 0.1 - 2'
    # input_folder = 'C:/Tim Mulder/CERNBox/Staff - TE-MPE-PE/Projects/ni-coils/geometry/output/FCS_750/results/result - 2024-07-31 - 0.1 - 21'

    ptr = plotter_v3(input_folder, xlim=[31.5, 34])


#!/usr/bin/env python3

import subprocess
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/test_geometry_1_output/', 'solver.yaml', 'geometry.yaml', 'solver_settings.yaml', 'circuits.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/test_geometry_1_1_output/', 'solver.yaml', 'geometry.yaml', 'solver_settings.yaml', 'circuits.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/test_geometry_4_1_output/', 'solver.yaml', 'geometry.yaml', 'solver_settings.yaml', 'circuits2.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/test_geometry_4_1_output/', 'solver.yaml', 'geometry.yaml', 'solver_settings.yaml', 'circuits1800.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/test_geometry_4_1_output/', 'solver.yaml', 'geometry.yaml', 'solver_settings.yaml', 'circuits300.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/test_geometry_4_1_output/', 'solver.yaml', 'geometry.yaml', 'solver_settings.yaml', 'circuits900.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/test_geometry_4_1_output/', 'solver.yaml', 'geometry.yaml', 'solver_settings.yaml', 'circuits2700.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/test_geometry_4_1_3600_output/', 'solver.yaml', 'geometry.yaml', 'solver_settings.yaml', 'circuits.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/test_geometry_4_output/', 'solver.yaml', 'geometry.yaml', 'solver_settings.yaml', 'circuits.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/test_geometry_4_MT_output/', 'solver.yaml', 'geometry.yaml', 'solver_settings2.yaml', 'circuits2.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/test_geometry_4_MT_output/', 'solver.yaml', 'geometry.yaml', 'solver_settings2.yaml', 'circuits3.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/test_geometry_12_1_output/', 'solver.yaml', 'geometry.yaml', 'solver_settings.yaml', 'circuits.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/test_geometry_12_9_output/', 'solver.yaml', 'geometry.yaml', 'solver_settings.yaml', 'circuits.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/test_geometry_10_output/', 'solver.yaml', 'geometry.yaml', 'solver_settings.yaml', 'circuits.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/test_geometry_10_output/', 'solver.yaml', 'geometry.yaml', 'solver_settings2.yaml', 'circuits2.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/AMS_100/', 'solver.yaml', 'geometry.yaml', 'solver_settings2.yaml', 'circuits2.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/test_geometry_18_1_out, 'solver.yaml', 'geometry.yaml', 'solver_settings.yaml', 'circuits.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/test_geometry_18_3_output/', 'solver.yaml', 'geometry.yaml', 'solver_settings.yaml', 'circuits.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/test_geometry_18_5_output/', 'solver.yaml', 'geometry.yaml', 'solver_settings.yaml', 'circuits.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/test_geometry_18_7_output/', 'solver.yaml', 'geometry.yaml', 'solver_settings.yaml', 'circuits.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/test_geometry_18_9_output/', 'solver.yaml', 'geometry.yaml', 'solver_settings.yaml', 'circuits.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/test_geometry_18_11_output/', 'solver.yaml', 'geometry.yaml', 'solver_settings.yaml', 'circuits.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/test_geometry_18_1_output/', 'solver.yaml', 'geometry.yaml', 'solver_settings.yaml', 'circuits2.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/test_geometry_18_3_output/', 'solver.yaml', 'geometry.yaml', 'solver_settings.yaml', 'circuits2.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/test_geometry_18_5_output/', 'solver.yaml', 'geometry.yaml', 'solver_settings.yaml', 'circuits2.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/test_geometry_18_7_output/', 'solver.yaml', 'geometry.yaml', 'solver_settings.yaml', 'circuits2.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/test_geometry_18_9_output/', 'solver.yaml', 'geometry.yaml', 'solver_settings.yaml', 'circuits2.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/test_geometry_18_11_output/', 'solver.yaml', 'geometry.yaml', 'solver_settings.yaml', 'circuits2.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/test_geometry_18_11_output/', 'solver.yaml', 'geometry.yaml', 'solver_settings.yaml', 'circuits3.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/test_geometry_19_2_output/', 'solver.yaml', 'geometry.yaml', 'solver_settings.yaml', 'circuits.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/test_geometry_19_2_output/', 'solver.yaml', 'geometry.yaml', 'solver_settings.yaml', 'circuits.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/test_geometry_19_2_output/', 'solver.yaml', 'geometry.yaml', 'solver_settings.yaml', 'circuits3.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/test_geometry_19_3_output/', 'solver.yaml', 'geometry.yaml', 'solver_settings2.yaml', 'circuits5.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/test_geometry_19_3_output/', 'solver.yaml', 'geometry.yaml', 'solver_settings.yaml', 'circuits4.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/test_geometry_19_3_output/', 'solver.yaml', 'geometry.yaml', 'solver_settings2.yaml', 'circuits2.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/test_geometry_23_output/', 'solver.yaml', 'geometry.yaml', 'solver_settings.yaml', 'circuits.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/test_geometry_23_2_output/', 'solver.yaml', 'geometry.yaml', 'solver_settings.yaml', 'circuits.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/test_geometry_23_3_output/', 'solver.yaml', 'geometry.yaml', 'solver_settings.yaml', 'circuits.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/test_geometry_23_3_output_debug/', 'solver.yaml', 'geometry.yaml', 'solver_settings.yaml', 'circuits.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/test_geometry_23_2_output/', 'solver.yaml', 'geometry.yaml', 'solver_settings2.yaml', 'circuits2.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/test_geometry_23_3_output/', 'solver.yaml', 'geometry.yaml', 'solver_settings2.yaml', 'circuits2.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/test_geometry_23_3_output/', 'solver.yaml', 'geometry.yaml', 'solver_settings_6h.yaml', 'circuits_300.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/test_geometry_23_3_5_output/', 'solver.yaml', 'geometry.yaml', 'solver_settings_6h.yaml', 'circuits_300.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/test_geometry_23_3_5_output/', 'solver.yaml', 'geometry.yaml', 'solver_settings_6h.yaml', 'circuits_900.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/test_geometry_23_3_5_output/', 'solver.yaml', 'geometry.yaml', 'solver_settings_6h.yaml', 'circuits_1800.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/test_geometry_23_3_5_output/', 'solver.yaml', 'geometry.yaml', 'solver_settings_6h.yaml', 'circuits_2700.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/test_geometry_23_3_5_output/', 'solver.yaml', 'geometry.yaml', 'solver_settings_6h.yaml', 'circuits_3600.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/test_geometry_23_4_output/', 'solver.yaml', 'geometry.yaml', 'solver_settings.yaml', 'circuits.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/test_geometry_23_5_output/', 'solver.yaml', 'geometry.yaml', 'solver_settings.yaml', 'circuits.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/test_geometry_23_6_output/', 'solver.yaml', 'geometry.yaml', 'solver_settings.yaml', 'circuits.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/test_geometry_23_6_output/', 'solver.yaml', 'geometry.yaml', 'solver_settings2.yaml', 'circuits2.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/test_geometry_4_MT_v2_output/', 'solver.yaml', 'geometry.yaml', 'solver_settings.yaml', 'circuits.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/test_geometry_24_1_output/', 'solver.yaml', 'geometry.yaml', 'solver_settings.yaml', 'circuits.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/40T_solenoid_1/', 'solver.yaml', 'geometry.yaml', 'solver_settings.yaml', 'circuits.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/40T_solenoid_3/', 'solver.yaml', 'geometry.yaml', 'solver_settings2.yaml', 'circuits2.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/valid_4_15_9/', 'solver.yaml', 'geometry.yaml', 'solver_settings.yaml', 'circuits.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/tree/', 'solver.yaml', 'geometry.yaml', 'solver_settings.yaml', 'circuits.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver_Light.py', 'geometry/output/test_geometry_25_output/', 'solver.yaml', 'geometry.yaml', 'solver_settings.yaml', 'circuits.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver_Light.py', 'geometry/output/test_geometry_25_output/', 'solver.yaml', 'geometry.yaml', 'solver_settings2.yaml', 'circuits2.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/test_geometry_15_output/', 'solver.yaml', 'geometry.yaml', 'solver_settings.yaml', 'circuits1.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/test_geometry_15_output/', 'solver.yaml', 'geometry.yaml', 'solver_settings.yaml', 'circuits2.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/test_geometry_15_output/', 'solver.yaml', 'geometry.yaml', 'solver_settings.yaml', 'circuits3.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/test_geometry_15_output/', 'solver.yaml', 'geometry.yaml', 'solver_settings.yaml', 'circuits4.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/test_geometry_15_1_output/', 'solver.yaml', 'geometry.yaml', 'solver_settings.yaml', 'circuits4.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/test_geometry_15_2_output/', 'solver.yaml', 'geometry.yaml', 'solver_settings.yaml', 'circuits4.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/test_geometry_15_3_output/', 'solver.yaml', 'geometry.yaml', 'solver_settings.yaml', 'circuits4.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver_Light.py', 'geometry/output/test_geometry_15_3_output/', 'solver.yaml', 'geometry.yaml', 'solver_settings.yaml', 'circuits4.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/test_geometry_15b_output/', 'solver.yaml', 'geometry.yaml', 'solver_settings.yaml', 'circuits1.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/test_geometry_15b_output/', 'solver.yaml', 'geometry.yaml', 'solver_settings.yaml', 'circuits2.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/test_geometry_15b_output/', 'solver.yaml', 'geometry.yaml', 'solver_settings.yaml', 'circuits3.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/test_geometry_15b_output/', 'solver.yaml', 'geometry.yaml', 'solver_settings.yaml', 'circuits4.yaml'])

# subprocess.call(['python', 'NI_Coil_Solver_Light.py', 'geometry/output/test_geometry_15_output/', 'solver.yaml', 'geometry.yaml', 'solver_settings.yaml', 'circuits1.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver_Light.py', 'geometry/output/test_geometry_15_output/', 'solver.yaml', 'geometry.yaml', 'solver_settings.yaml', 'circuits1.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver_Light.py', 'geometry/output/test_geometry_15_1_output/', 'solver.yaml', 'geometry.yaml', 'solver_settings.yaml', 'circuits1.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/test_geometry_15_1_output/', 'solver.yaml', 'geometry.yaml', 'solver_settings.yaml', 'circuits1.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver_Light.py', 'geometry/output/test_geometry_27_output/', 'solver.yaml', 'geometry.yaml', 'solver_settings.yaml', 'circuits1.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/test_geometry_27_output/', 'solver.yaml', 'geometry.yaml', 'solver_settings.yaml', 'circuits1.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/test_geometry_27_1_output/', 'solver.yaml', 'geometry.yaml', 'solver_settings.yaml', 'circuits1.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/test_geometry_27_1_output/', 'solver.yaml', 'geometry.yaml', 'solver_settings.yaml', 'circuits2.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/test_geometry_27_1_output/', 'solver.yaml', 'geometry.yaml', 'solver_settings.yaml', 'circuits3.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/test_geometry_27_1_output/', 'solver.yaml', 'geometry.yaml', 'solver_settings.yaml', 'circuits4.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver_Light.py', 'geometry/output/test_geometry_27_insulated_output/', 'solver.yaml', 'geometry.yaml', 'solver_settings.yaml', 'circuits4.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/test_geometry_27_output/', 'solver.yaml', 'geometry.yaml', 'solver_settings.yaml', 'circuits1.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/test_geometry_27_output/', 'solver.yaml', 'geometry.yaml', 'solver_settings.yaml', 'circuits2.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/test_geometry_27_output/', 'solver.yaml', 'geometry.yaml', 'solver_settings.yaml', 'circuits3.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/test_geometry_27_output/', 'solver.yaml', 'geometry.yaml', 'solver_settings.yaml', 'circuits4.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/test_geometry_27_2_output/', 'solver.yaml', 'geometry.yaml', 'solver_settings.yaml', 'circuits2.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/test_geometry_27_2_output/', 'solver.yaml', 'geometry.yaml', 'solver_settings.yaml', 'circuits3.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/test_geometry_27_2_output/', 'solver.yaml', 'geometry.yaml', 'solver_settings.yaml', 'circuits4.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver_Light.py', 'geometry/output/test_geometry_27_2_output/', 'solver.yaml', 'geometry.yaml', 'solver_settings.yaml', 'circuits1.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver_Light.py', 'geometry/output/test_geometry_27_2_output/', 'solver.yaml', 'geometry.yaml', 'solver_settings.yaml', 'circuits2.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver_Light.py', 'geometry/output/test_geometry_27_2_output/', 'solver.yaml', 'geometry.yaml', 'solver_settings.yaml', 'circuits3.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver_Light.py', 'geometry/output/test_geometry_27_2_output/', 'solver.yaml', 'geometry.yaml', 'solver_settings.yaml', 'circuits4.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver_Light.py', 'geometry/output/test_geometry_23_FCS_output/', 'solver.yaml', 'geometry.yaml', 'solver_settings_6h.yaml', 'circuits_1800.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/test_geometry_23_FCS_output/', 'solver.yaml', 'geometry.yaml', 'solver_settings_6h_pid.yaml', 'circuits_1800_PID.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/test_geometry_23_FCS_output/', 'solver.yaml', 'geometry.yaml', 'solver_settings_6h_pid_2.yaml', 'circuits_1800_PID.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/test_geometry_23_FCS_output_new/', 'solver.yaml', 'geometry.yaml', 'solver_settings_6h_pid_2.yaml', 'circuits_1800_PID.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/test_geometry_23_FCS_2_output_new/', 'solver.yaml', 'geometry.yaml', 'solver_settings_6h_pid_2.yaml', 'circuits_1800_PID.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/test_geometry_23_FCS_c_output/', 'solver.yaml', 'geometry.yaml', 'solver_settings_6h_pid.yaml', 'circuits_1800_PID.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/test_geometry_23_FCS_d_output/', 'solver.yaml', 'geometry.yaml', 'solver_settings_6h_pid.yaml', 'circuits_2700_PID.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/test_geometry_23_FCS_e_output/', 'solver.yaml', 'geometry.yaml', 'solver_settings_6h_pid2.yaml', 'circuits_2700_PID.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/test_geometry_28_FCS/', 'solver.yaml', 'geometry.yaml', 'solver_settings_6h_pid2.yaml', 'circuits_2700_PID.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/test_geometry_28_FCS/', 'solver.yaml', 'geometry.yaml', 'solver_quench.yaml', 'quench.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/test_geometry_28_FCS/', 'solver.yaml', 'geometry.yaml', 'solver_quench.yaml', 'quench4.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/test_geometry_28_FCS_1/', 'solver.yaml', 'geometry.yaml', 'solver_settings_6h_pid2.yaml', 'circuits_2700_PID.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/FCS_750/', 'solver.yaml', 'geometry.yaml', 'solver_settings_6h_pid2.yaml', 'circuits_2700_PID.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver_Torch.py', 'geometry/output/FCS_375_Torch/', 'solver.yaml', 'geometry.yaml', 'solver_settings_6h_pid2.yaml', 'circuits_2700.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver_Torch.py', 'geometry/output/FCS_750_Torch/', 'solver.yaml', 'geometry.yaml', 'solver_settings_6h_pid3.yaml', 'circuits_2700.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver_Torch.py', 'geometry/output/FCS_750_Torch_1/', 'solver.yaml', 'geometry.yaml', 'solver_settings_6h_pid3.yaml', 'circuits_2700.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver_Torch.py', 'geometry/output/FCS_750_Torch/', 'solver.yaml', 'geometry.yaml', 'solver_quench.yaml', 'quench1.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver_Torch.py', 'geometry/output/FCS_750_Torch/', 'solver.yaml', 'geometry.yaml', 'solver_quench.yaml', 'quench2.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver_Torch.py', 'geometry/output/FCS_750_Torch/', 'solver.yaml', 'geometry.yaml', 'solver_quench.yaml', 'quench3.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver_Torch.py', 'geometry/output/FCS_375_Torch/', 'solver.yaml', 'geometry.yaml', 'solver_quench.yaml', 'quench1.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver_Torch.py', 'geometry/output/FCS_750_Torch_1/', 'solver.yaml', 'geometry.yaml', 'solver_stable.yaml', 'stable1.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver_Torch.py', 'geometry/output/CD_small_coils/', 'solver.yaml', 'geometry.yaml', 'solver_settings.yaml', 'circuits.yaml'])
subprocess.call(['python', 'NI_Coil_Solver_Torch.py', 'geometry/output/CD_small_coils/', 'solver.yaml', 'geometry.yaml', 'solver_quench.yaml', 'quench1.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver_Light.py', 'geometry/output/36_insulated_gianluca/', 'solver.yaml', 'geometry.yaml', 'solver_settings.yaml', 'ramp1.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/FCS_375/', 'solver.yaml', 'geometry.yaml', 'solver_settings_6h_pid3.yaml', 'circuits_2700.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/FCS_750/', 'solver.yaml', 'geometry.yaml', 'solver_quench.yaml', 'quench1.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/FCS_750/', 'solver.yaml', 'geometry.yaml', 'solver_quench.yaml', 'quench2.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/FCS_750/', 'solver.yaml', 'geometry.yaml', 'solver_quench.yaml', 'quench3.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/FCS_750/', 'solver.yaml', 'geometry.yaml', 'solver_quench.yaml', 'quench5.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/FCS_750/', 'solver.yaml', 'geometry.yaml', 'solver_quench.yaml', 'quench6.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/FCS_750/', 'solver.yaml', 'geometry.yaml', 'solver_quench.yaml', 'quench7.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/FCS_750/', 'solver.yaml', 'geometry.yaml', 'solver_quench.yaml', 'quench8.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/FCS_375/', 'solver.yaml', 'geometry.yaml', 'solver_quench.yaml', 'quench1.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/FCS_375/', 'solver.yaml', 'geometry.yaml', 'solver_quench.yaml', 'quench2.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/FCS_375/', 'solver.yaml', 'geometry.yaml', 'solver_quench.yaml', 'quench3.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/FCS_375/', 'solver.yaml', 'geometry.yaml', 'solver_quench.yaml', 'quench5.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/FCS_375/', 'solver.yaml', 'geometry.yaml', 'solver_quench.yaml', 'quench6.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/FCS_375/', 'solver.yaml', 'geometry.yaml', 'solver_quench.yaml', 'quench7.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/FCS_750/', 'solver.yaml', 'geometry.yaml', 'solver_quench.yaml', 'quench.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/FCS_375/', 'solver.yaml', 'geometry.yaml', 'solver_settings_6h_pid2.yaml', 'circuits_2700_PID.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/FCS_from_EXCEL_low_out/', 'solver.yaml', 'geometry.yaml', 'solver_settings_6h_pid2.yaml', 'circuits_2700_PID_thermal.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/test_geometry_29_AMS_100_v3/', 'solver.yaml', 'geometry.yaml', 'solver_settings.yaml', 'circuits.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/6D_B7_NI/', 'solver.yaml', 'geometry.yaml', 'solver_settings.yaml', 'circuits.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/test_geometry_23_FCS_output/', 'solver.yaml', 'geometry.yaml', 'solver_settings_6h_pid.yaml', 'circuits_2700_PID.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/test_geometry_23_FCS_output/', 'solver.yaml', 'geometry.yaml', 'solver_settings.yaml', 'circuits2.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver.py', 'geometry/output/test_geometry_23_thermal_output/', 'solver.yaml', 'geometry.yaml', 'solver_settings_6h_pid.yaml', 'circuits_1800_PID.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver_Light.py', 'geometry/output/test_geometry_15_output/', 'solver.yaml', 'geometry.yaml', 'solver_settings.yaml', 'circuits2.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver_Light.py', 'geometry/output/test_geometry_15_output/', 'solver.yaml', 'geometry.yaml', 'solver_settings.yaml', 'circuits3.yaml'])
# subprocess.call(['python', 'NI_Coil_Solver_Light.py', 'geometry/output/test_geometry_15_output/', 'solver.yaml', 'geometry.yaml', 'solver_settings.yaml', 'circuits4.yaml'])



#!/usr/bin/env python3

import numpy as np
import matplotlib.pyplot as plt
from 	matplotlib.animation 	import FuncAnimation
from    matplotlib              import cm
import pandas as pd
import yaml
import pickle
import os
from    mpl_toolkits.axes_grid1 import make_axes_locatable

class plotter_v2:
    # This plotter will only be used to create gifs from the Paraview output files

    # paths
    rpath = ''
    input2 = None
    parameter = None
    time_unit = 'ms'
    current_input = None

    fig = plt.figure(figsize=(6,5)) # plt.figure(figsize=(8,6)) # AMS 14,5
    ax = fig.add_subplot(111)
    plot_every= 1

    # Data
    input_data = None
    limits = None

    colormap = cm.seismic #cm.gist_heat #cm.YlGn

    # no_title = False
    # no_bars = False
    # no_axis = False
    # custom_title = None

    # Grid EUCAS BERNARDO
    # xmin= 0.0
    # xmax= 0.3
    # ymin= -0.5
    # ymax= 0.5
    # nx= 30
    # ny= 60

    # # Grid EUCAS BERNARDO
    # xmin= 0.0
    # xmax= 0.3
    # ymin= -0.4
    # ymax= 0.4
    # nx= 30
    # ny= 60

    # # Grid AMS
    # xmin = 0.0
    # xmax = 8.0
    # ymin = -5
    # ymax = 5
    # nx = 20
    # ny = 40
    # Bmin = 0.0
    # Bmax = 0.8

    # Grid 40T Solenoid
    xmin = 0.0
    xmax = 0.3
    # ymin = -0.4
    # ymax = 0.4
    ymin = -0.5
    ymax = 0.5
    nx = 30
    ny = 60
    # Bmin = 0.0
    # Bmax = 45
    #
    # # Grid tree
    # xmin = 0.0
    # xmax = 0.3
    # ymin = -0.4
    # ymax = 0.4
    # nx = 30
    # ny = 80
    Bmin = 0.0
    Bmax = 20.0

    rotate_90 = False

    custom_title = None

    no_title = False
    no_bars = False
    no_axis = True

    def __init__(self, rpath, parameter='', input2 = None, time_unit = 'ms', current_input='NICQS', rot90 = False, Bmax=20.0):
        self.rpath = rpath
        self.input2 = input2
        self.parameter = parameter
        self.time_unit = time_unit
        self.current_input = current_input
        self.rotate_90 = rot90
        self.Bmax=Bmax

        self.load_yaml()
        self.load_current()
        self.initiate_figure()

        self.make_gif()

    def initiate_figure(self):
        if self.rotate_90:
            self.figsize = (2 + 6 / (2 * self.xmax / (self.ymax - self.ymin)), 6)
        else:
            self.figsize = (8, 6 / (2 * self.xmax / (self.ymax - self.ymin)))
        self.fig = plt.figure(figsize=self.figsize)
        self.ax = self.fig.add_subplot(111)
        self.axb = self.ax.twinx()

    def load_yaml(self):
        # Reads the contents of the yaml file
        if self.current_input == 'NICQS':
            with open(self.rpath + os.sep + 'Paraview' + os.sep + 'python_files_to_load_' + self.parameter + '.yaml') as file:
                data = yaml.load(file, Loader=yaml.FullLoader)
                self.input_data = data['data']
                self.limits = data['limits']

        with open(self.input2 + os.sep + 'geometry.yaml') as file:
            data = yaml.load(file, Loader=yaml.FullLoader)
            self.positions_outer = data['positions_outer']
            grid_information = data['grid_information']
            # Grid tree
            self.xmin = grid_information[0]
            self.xmax = grid_information[1]
            self.ymin = grid_information[2]
            self.ymax = grid_information[3]
            self.nx = grid_information[4]
            self.ny = grid_information[5]


    def load_current(self):
        if self.current_input == 'Xyce':
            self.current_II_values = pd.read_csv(self.rpath + os.sep + 'currents.csv', delimiter=',', on_bad_lines='skip')
            self.len_entries = len(self.current_II_values)
            self.index_values = np.arange(0, self.len_entries, int(np.ceil(self.len_entries / 100)))  # TODO GHETTO FIX
            self.time_steps = self.current_II_values['TIME'].values[self.index_values]
        else:
            self.current_II_values = pd.read_csv(self.rpath + os.sep + 'current.csv')
            self.len_entries = len(self.current_II_values)
            # self.index_values = np.arange(0, self.len_entries, int(np.ceil(self.len_entries/len(list(self.input_data.keys()))))) # TODO GHETTO FIX
            self.index_values = [i for i in takespread(np.arange(self.len_entries), len(list(self.input_data.keys())))]
            self.time_steps = list(self.input_data.keys())

            n = 0
            for i in self.index_values:
                print(self.current_II_values.iloc[i].values[0])
                print(list(self.input_data.keys())[n])
                n+=1

        # Loading fieldmaps
        path = self.input2 + os.sep + 'Bz_full.pkl'
        try:
            with open(path, 'rb') as fin:
                self.Bz_full = pickle.load(fin)
        except OSError:
            print("Loading of %s has failed." % (path))
        else:
            print("Successfully loaded %s" % (path))
        path = self.input2 + os.sep + 'Br_full.pkl'
        try:
            with open(path, 'rb') as fin:
                self.Br_full = pickle.load(fin)
        except OSError:
            print("Loading of %s has failed." % (path))
        else:
            print("Successfully loaded %s" % (path))

        if self.current_input == 'Xyce':
            path = self.rpath + os.sep + 'current.pkl'
            try:
                with open(path, 'rb') as fin:
                    self.I = pickle.load(fin)
            except OSError:
                print("Loading of %s has failed." % (path))
            else:
                print("Successfully loaded %s" % (path))

            n = self.Bz_full.shape[1]
            self.columnlist = ['I(L' + str(i) +')' for i in range(n)]



    def make_gif(self):

        try:
            os.mkdir(self.rpath + os.sep  + 'GIFs')
        except OSError:
            print("Creation of the directory %s failed" % self.rpath + os.sep  + 'GIFs')
        else:
            print("Successfully created the directory %s " % self.rpath + os.sep  + 'GIFs')



        # self.limits['minmax'][1] = 0.5 * self.limits['minmax'][1]

        # Plot the magnetic field:
        if self.current_input == 'NICQS':
            current = self.current_II_values.iloc[0].values[1:]
            self.time_steps = list(self.input_data.keys())
            print(len(self.time_steps), self.time_steps)
            step_data = self.input_data[self.time_steps[4]]
        elif self.current_input == 'Xyce':
            self.current_II_values = self.current_II_values[self.columnlist]
            current = self.current_II_values.iloc[0].values

        Bz = np.dot(self.Bz_full, current)
        Br = np.dot(self.Br_full, current)

        if self.rotate_90:
            self.xv, self.yv = np.meshgrid(np.linspace(self.ymin, self.ymax, self.ny), np.linspace(-self.xmax, self.xmax, 2*self.nx-1))
            CS1 = self.ax.contourf(np.linspace(self.ymin, self.ymax, self.ny), np.linspace(self.xmin, self.xmax, self.nx), np.rot90(((Br**2 + Bz**2) ** 0.5).reshape((self.ny, self.nx))), cmap=cm.coolwarm, levels=np.linspace(0, self.Bmax, 21), vmin=self.Bmin, vmax=self.Bmax)

            if self.current_input == 'Xyce':
                for i in range(len(self.positions_outer)):
                    [self.axb.plot(j[1], -1 * np.array(j[0]), color='black') for j in self.positions_outer[i]]
                    [self.axb.plot(j[1], j[0], color='black') for j in self.positions_outer[i]]
        else:
            self.xv, self.yv = np.meshgrid(np.linspace(-self.xmax, self.xmax, (2 * self.nx) - 1), np.linspace(self.ymin, self.ymax, self.ny))
            CS1 = self.ax.contourf(np.linspace(self.xmin, self.xmax, self.nx), np.linspace(self.ymin, self.ymax, self.ny), ((Br ** 2 + Bz ** 2) ** 0.5).reshape((self.ny, self.nx)), cmap=cm.coolwarm, levels=np.linspace(0, self.Bmax, 21), vmin=self.Bmin, vmax=self.Bmax)
            if self.current_input == 'Xyce':
                for i in range(len(self.positions_outer)):
                    [self.axb.plot(-1 * np.array(j[0]), j[1], color='black') for j in self.positions_outer[i]]
                    [self.axb.plot(j[0], j[1], color='black') for j in self.positions_outer[i]]

        if self.parameter in ['Fz_density', 'Fr_density']:
            self.limits['minmax'][0] *= 1e-9
            self.limits['minmax'][1] *= 1e-9

        if self.current_input == 'NICQS':
            for i in step_data:
                df = pd.read_csv(i)
                if self.parameter in ['Fz_density', 'Fr_density']:
                    # Going from N/m3 to N/mm3, purely because the values were too large (>1e10) and the arrays need to be cast to an int32 array.
                    # Max value of int32 is 2,147,483,647.
                    df['v'] *= 1e-9
                self.ax.tricontourf(df['x'].values, df['y'].values, df['v'].values, vmin=self.limits['minmax'][0], vmax=self.limits['minmax'][1], cmap=self.colormap)
                CS2 = self.ax.tricontourf(-1*df['x'].values, df['y'].values, df['v'].values, levels = np.linspace(self.limits['minmax'][0], self.limits['minmax'][1], 21), vmin=self.limits['minmax'][0], vmax=self.limits['minmax'][1], cmap=self.colormap)

        divider1 = make_axes_locatable(self.ax)
        # cax1 = divider1.append_axes('right', size='10%', pad=0.05)
        # cax2 = divider1.append_axes('right', size='10%', pad=0.05)

        if not self.no_bars:
            cbar = self.fig.colorbar(CS1, orientation='vertical')
            cbar.set_label('Magnetic Field, T')
            if self.current_input == 'NICQS':
                cbar2 = self.fig.colorbar(CS2, orientation='vertical', location='left')
                if self.parameter == 'T':
                    cbar2.set_label('Temperature, K')
                elif self.parameter == 'I_density':
                    cbar2.set_label('Current Density, A/mm$^2$')
                elif self.parameter in ['Fz_density', 'Fr_density']:
                    cbar2.set_label('Force Density, N/mm$^3$')

        if self.rotate_90:
            self.ax.set_ylim((-self.xmax, self.xmax))
            self.ax.set_xlim((self.ymin, self.ymax))
        else:
            self.ax.set_xlim((-self.xmax, self.xmax))
            self.ax.set_ylim((self.ymin, self.ymax))

        if not self.no_axis:
            if self.rotate_90:
                self.ax.set_aspect('equal', 'box')
                self.ax.set_xlabel('Axial position, m')
                self.ax.set_ylabel('Radial position, m')
            else:
                self.ax.set_aspect('equal', 'box')
                self.ax.set_xlabel('Radial position, m')
                self.ax.set_ylabel('Axial position, m')
            # self.fig.tight_layout()
        else:
            self.ax.tick_params(left = False, right = False , labelleft = False ,
                labelbottom = False, bottom = False)

        if self.custom_title is not None:
            self.ax.set_title(self.custom_title)

        print(self.time_steps)
        anim = FuncAnimation(self.fig, self.update_gif, frames=np.arange(0, len(self.time_steps), self.plot_every), interval=100)
        # IF YOU GET AN ERROR IN SAVING THE GIF -> DOWNLOAD IMAGEMAGICK
        # anim.save(fname, dpi=120, writer='imagemagick', fps=20)
        fname = self.rpath + os.sep  + 'GIFs' + os.sep + 'test_' + self.parameter + '.gif'
        anim.save(fname, dpi=100, fps=10)
        plt.close('all')

    def update_gif(self, i):
        # Updates the gif
        self.ax.clear()
        # self.axb.clear()

        print('Frame', i)

        if self.current_input == 'NICQS':
            step_data = self.input_data[self.time_steps[i]]
            current = self.current_II_values.iloc[self.index_values[i]].values[1:]
        elif self.current_input == 'Xyce':
            current = self.current_II_values.iloc[self.index_values[i]].values * self.I #TODO what is this self.I
        Bz = np.dot(self.Bz_full, current)
        Br = np.dot(self.Br_full, current)

        Bz = np.transpose(Bz.reshape((self.nx, self.ny)))
        Br = np.transpose(Br.reshape((self.nx, self.ny)))

        Bz = np.hstack([np.fliplr(Bz[:, 1:]), Bz])
        Br = np.hstack([-1 * np.fliplr(Br[:, 1:]), Br])

        print(np.max(((Br ** 2 + Bz ** 2) ** 0.5)))

        if self.rotate_90:
            self.ax.contourf(np.linspace(self.ymin, self.ymax, self.ny), np.linspace(-self.xmax, self.xmax, (2 * self.nx) - 1),
                             np.rot90((Br ** 2 + Bz ** 2) ** 0.5), cmap=cm.coolwarm,
                             levels=np.linspace(self.Bmin, self.Bmax, 21), vmin=self.Bmin, vmax=self.Bmax)

            self.ax.streamplot(self.xv, self.yv, -1*np.rot90(Bz), np.rot90(Br), density=2.0, linewidth=0.5, color='white')
            # for k in range(len(self.positions_outer)):
            #     [self.axb.plot(j[1], -1 * np.array(j[0]), color='black') for j in self.positions_outer[k]]
            #     [self.axb.plot(j[1], j[0], color='black') for j in self.positions_outer[k]]
        else:
            self.ax.contourf(np.linspace(-self.xmax, self.xmax, (2*self.nx)-1), np.linspace(self.ymin, self.ymax, self.ny),
                             ((Br ** 2 + Bz ** 2) ** 0.5), cmap=cm.coolwarm,
                             levels=np.linspace(self.Bmin, self.Bmax, 21), vmin=self.Bmin, vmax=self.Bmax)

            self.ax.streamplot(self.xv, self.yv, Br, Bz, density=2.0, linewidth=0.5, color='white')

        if self.current_input == 'NICQS':
            if self.rotate_90:
                for j in step_data:
                    df = pd.read_csv(j)
                    if self.parameter in ['Fz_density', 'Fr_density']:
                        # Going from N/m3 to N/mm3, purely because the values were too large (>1e10) and the arrays need to be cast to an int32 array.
                        # Max value of int32 is 2,147,483,647.
                        df['v'] *= 1e-9

                    self.ax.tricontourf(df['y'].values, df['x'].values, df['v'].values, levels = np.linspace(self.limits['minmax'][0], self.limits['minmax'][1], 21), cmap=self.colormap)#vmin=self.limits['minmax'][0], vmax=self.limits['minmax'][1], cmap=cm.gist_heat)
                    self.ax.tricontourf(df['y'].values, -1*df['x'].values, df['v'].values, levels = np.linspace(self.limits['minmax'][0], self.limits['minmax'][1], 21), cmap=self.colormap)#vmin=self.limits['minmax'][0], vmax=self.limits['minmax'][1], cmap=cm.gist_heat)
            else:
                for j in step_data:
                    df = pd.read_csv(j)
                    # print(i, j, np.min(df['v'].values), np.max(df['v'].values))
                    self.ax.tricontourf(df['x'].values, df['y'].values, df['v'].values, levels = np.linspace(self.limits['minmax'][0], self.limits['minmax'][1], 21), cmap=self.colormap)#vmin=self.limits['minmax'][0], vmax=self.limits['minmax'][1], cmap=cm.gist_heat)
                    self.ax.tricontourf(-1*df['x'].values, df['y'].values, df['v'].values, levels = np.linspace(self.limits['minmax'][0], self.limits['minmax'][1], 21), cmap=self.colormap)#vmin=self.limits['minmax'][0], vmax=self.limits['minmax'][1], cmap=cm.gist_heat)

        if not self.no_title:
            if self.time_unit == 'ms':
                self.ax.set_title('t = ' + str(np.round((self.time_steps[i]-self.time_steps[0]) * 1e3)) + ' ms')
            elif self.time_unit == 's':
                self.ax.set_title('t = ' + str(np.round((self.time_steps[i]-self.time_steps[0]))) + ' s')
            elif self.time_unit == 'h':
                t = (self.time_steps[i]-self.time_steps[0]) / 3600
                self.ax.set_title('t = ' + str(int(t)) + 'h' + str(int(t*60)%60) + 'm')

        if self.custom_title is not None:
            self.ax.set_title(self.custom_title)

        if self.rotate_90:
            self.ax.set_ylim((-self.xmax, self.xmax))
            self.ax.set_xlim((self.ymin, self.ymax))
            self.axb.set_ylim((-self.xmax, self.xmax))
            self.axb.set_xlim((self.ymin, self.ymax))
        else:
            self.ax.set_xlim((-self.xmax, self.xmax))
            self.ax.set_ylim((self.ymin, self.ymax))
            self.axb.set_xlim((-self.xmax, self.xmax))
            self.axb.set_ylim((self.ymin, self.ymax))
        # self.ax.set_aspect('equal', 'box')

        self.fig.savefig(self.rpath + os.sep  + 'GIFs' + os.sep + self.parameter + '_' + str(i) + '.png', dpi=300)

def takespread(sequence, num):
    length = float(len(sequence))
    for i in range(num):
        yield sequence[int(np.ceil(i * length / num))]

if __name__ == "__main__":
        print('No input yaml file provided, using the NI_Coil_Quench_2D_input.yaml file.')
        # input1 = 'C:/Tim Mulder/CERNBox/Staff - TE-MPE-PE/Projects/ni-coils/geometry/output/test_geometry_23_2_output/results/result - 2023-08-21 - 0.1 - 3'
        # input2 = 'C:/Tim Mulder/CERNBox/Staff - TE-MPE-PE/Projects/ni-coils/geometry/output/test_geometry_23_2_output'
        # input1 = 'C:/Tim Mulder/CERNBox/Staff - TE-MPE-PE/Projects/ni-coils/geometry/output/test_geometry_4_MT_output/results/result - 2023-09-04 - 0.1 - 1'
        # input2 = 'C:/Tim Mulder/CERNBox/Staff - TE-MPE-PE/Projects/ni-coils/geometry/output/test_geometry_4_MT_output'
        # input1 = 'C:/Tim Mulder/CERNBox/Staff - TE-MPE-PE/Projects/ni-coils/geometry/output/test_geometry_4_MT_output/results/result - 2023-09-08 - 0.1 - 5'
        # input2 = 'C:/Tim Mulder/CERNBox/Staff - TE-MPE-PE/Projects/ni-coils/geometry/output/test_geometry_4_MT_output'
        # input1 = 'C:/Tim Mulder/CERNBox/Staff - TE-MPE-PE/Projects/ni-coils/geometry/output/test_geometry_23_2_output/results/result - 2023-10-09 - 0.1 - 1'
        # input2 = 'C:/Tim Mulder/CERNBox/Staff - TE-MPE-PE/Projects/ni-coils/geometry/output/test_geometry_23_2_output'
        # input1 = 'C:/Tim Mulder/CERNBox/Staff - TE-MPE-PE/Projects/ni-coils/geometry/output/AMS_100/results/result - 2023-12-05 - 0.1 - 7/'
        # input2 = 'C:/Tim Mulder/CERNBox/Staff - TE-MPE-PE/Projects/ni-coils/geometry/output/AMS_100/'
        # input1 = 'C:/Tim Mulder/CERNBox/Staff - TE-MPE-PE/Projects/ni-coils/geometry/output/tree/results/result - 2023-12-05 - 0.1 - 3/'
        # input2 = 'C:/Tim Mulder/CERNBox/Staff - TE-MPE-PE/Projects/ni-coils/geometry/output/tree/'
        # input1 = 'C:/Tim Mulder/CERNBox/Staff - TE-MPE-PE/Projects/ni-coils/geometry/output/test_geometry_23_FCS_output/results/result - 2024-03-09 - 0.1 - 3/'
        # input2 = 'C:/Tim Mulder/CERNBox/Staff - TE-MPE-PE/Projects/ni-coils/geometry/output/test_geometry_23_FCS_output/
        # input1 = 'C:/Tim Mulder/CERNBox/Staff - TE-MPE-PE/Projects/ni-coils/geometry/output/test_geometry_23_FCS_c_output/results/result - 2024-03-26 - 0.1 - 1/'
        # input2 = 'C:/Tim Mulder/CERNBox/Staff - TE-MPE-PE/Projects/ni-coils/geometry/output/test_geometry_23_FCS_c_output/'
        # input1 = 'C:/Tim Mulder/CERNBox/Staff - TE-MPE-PE/Projects/ni-coils/geometry/output/test_geometry_23_FCS_e_output/results/result - 2024-04-02 - 0.1 - 1/'
        # input2 = 'C:/Tim Mulder/CERNBox/Staff - TE-MPE-PE/Projects/ni-coils/geometry/output/test_geometry_23_FCS_e_output/'
        # input1 = 'C:/Tim Mulder/CERNBox/Staff - TE-MPE-PE/Projects/Muon Collider Final Cooling/Simulations/6D Cooling - Netlists'
        # input2 = 'C:/Tim Mulder/CERNBox/Staff - TE-MPE-PE/Projects/ni-coils/geometry/output/test_geometry_6D_cooling_B7_output'
        # input1 = 'C:/Tim Mulder/CERNBox/Staff - TE-MPE-PE/Projects/ni-coils/geometry/output/test_geometry_28_FCS/results/result - 2024-05-28 - 0.1 - 4/'
        # input2 = 'C:/Tim Mulder/CERNBox/Staff - TE-MPE-PE/Projects/ni-coils/geometry/output/test_geometry_28_FCS/'
        # input1 = 'C:/Tim Mulder/CERNBox/Staff - TE-MPE-PE/Projects/ni-coils/geometry/output/test_geometry_29_AMS_100_v2/results/result - 2024-07-03 - 0.1 - 5/'
        # input2 = 'C:/Tim Mulder/CERNBox/Staff - TE-MPE-PE/Projects/ni-coils/geometry/output/test_geometry_29_AMS_100_v2/'
        # input1 = 'C:/Tim Mulder/CERNBox/Staff - TE-MPE-PE/Projects/ni-coils/geometry/output/test_geometry_29_AMS_100_v3/results/result - 2024-09-23 - 0.1 - 2/'
        # input2 = 'C:/Tim Mulder/CERNBox/Staff - TE-MPE-PE/Projects/ni-coils/geometry/output/test_geometry_29_AMS_100_v3/'
        # input1 = 'C:/Tim Mulder/CERNBox/Staff - TE-MPE-PE/Projects/ni-coils/geometry/output/FCS_750_Torch_1/results/result - 2025-05-09 - Quench6 - 5/'
        # input2 = 'C:/Tim Mulder/CERNBox/Staff - TE-MPE-PE/Projects/ni-coils/geometry/output/FCS_750_Torch_1/'
        # input1 = 'C:/Tim Mulder/CERNBox/Staff - TE-MPE-PE/Projects/ni-coils/geometry/output/muon_6D_demonstrator_v24/results/result - 2025-07-21 - ramp-100 - 9'
        # input2 = 'C:/Tim Mulder/CERNBox/Staff - TE-MPE-PE/Projects/ni-coils/geometry/output/muon_6D_demonstrator_v24/'
        input1 = 'C:/Tim Mulder/CERNBox/Staff - TE-MPE-PE/Projects/ni-coils/geometry/output/muon_6D_demonstrator_v24_lattice/results/result - 2025-07-21 - 2'
        input2 = 'C:/Tim Mulder/CERNBox/Staff - TE-MPE-PE/Projects/ni-coils/geometry/output/muon_6D_demonstrator_v24_lattice/'
        # input1 = 'C:/Tim Mulder/CERNBox/Staff - TE-MPE-PE/Projects/ni-coils/geometry/output/muon_6D_demonstrator_lattice_copper/results/result - 2025-05-05 - 2'
        # input2 = 'C:/Tim Mulder/CERNBox/Staff - TE-MPE-PE/Projects/ni-coils/geometry/output/muon_6D_demonstrator_lattice_copper/'
        # input1 = 'C:/Tim Mulder/CERNBox/Staff - TE-MPE-PE/Projects/ni-coils/geometry/output/muon_6D_demonstrator_lattice_copper2/results/result - 2025-05-07 - 1'
        # input2 = 'C:/Tim Mulder/CERNBox/Staff - TE-MPE-PE/Projects/ni-coils/geometry/output/muon_6D_demonstrator_lattice_copper2/'
        # input1 = 'C:/Tim Mulder/CERNBox/Staff - TE-MPE-PE/Projects/ni-coils/geometry/output/muon_6D_demonstrator_lattice_copper3/results/result - 2025-05-06 - 1'
        # input2 = 'C:/Tim Mulder/CERNBox/Staff - TE-MPE-PE/Projects/ni-coils/geometry/output/muon_6D_demonstrator_lattice_copper3/'

        # ptr = plotter_v2(input1, 'I_density', input2 = input2, time_unit = 's', current_input = 'Xyce', rot90=True, Bmax=15.0)
        # ptr = plotter_v2(input1, 'T', input2 = input2, time_unit = 'ms', rot90=True, Bmax=45.0)
        # ptr = plotter_v2(input1, 'I_density', input2 = input2, time_unit = 's', rot90=True, Bmax=16)
        ptr = plotter_v2(input1, 'Fz_density', input2 = input2, time_unit = 'ms', rot90=True, Bmax=16)


#!/usr/bin/env python3

import subprocess
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/test_geometry_1.yaml', 'test_geometry_1_output'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/test_geometry_1_1.yaml', 'test_geometry_1_1_output'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/test_geometry_2.yaml', 'test_geometry_2_output'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/test_geometry_3.yaml', 'test_geometry_3_output'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/test_geometry_4.yaml', 'test_geometry_4_output'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/test_geometry_4_MT.yaml', 'test_geometry_4_MT_output'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/test_geometry_4_MT_Cu1.yaml', 'test_geometry_4_MT_Cu1_output'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/test_geometry_4_MT_v2.yaml', 'test_geometry_4_MT_v2_output'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/test_geometry_4_1.yaml', 'test_geometry_4_1_output'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/test_geometry_4_1_4mm.yaml', 'test_geometry_4_1_output_4mm'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yamlt_geometry_4_single.yaml', 'test_geometry_4_single_output'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/test_geometry_4_single_4mm.yaml', 'test_geometry_4_single_output_4mm'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/test_geometry_4_1_3600.yaml', 'test_geometry_4_1_3600_output'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/test_geometry_5.yaml', 'test_geometry_5_output'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/test_geometry_6_AMS_100.yaml', 'test_geometry_6_output'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/test_geometry_7_AMS_100.yaml', 'test_geometry_7_output'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/test_geometry_8.yaml', 'test_geometry_8_output'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/test_geometry_10_AMS_100.yaml', 'AMS_100'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/test_geometry_11_presentation.yaml', 'test_geometry_11_output'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/test_geometry_12_screening_currents.yaml', 'test_geometry_12_output'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/test_geometry_12_1_screening_currents.yaml', 'test_geometry_12_1_output'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/test_geometry_12_9_screening_currents.yaml', 'test_geometry_12_9_output'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/test_geometry_13_Eriks_test.yaml', 'test_geometry_13_output'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/test_geometry_14_screening_currents.yaml', 'test_geometry_14_output'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/test_geometry_15_screening_currents.yaml', 'test_geometry_15_output'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/test_geometry_15b_screening_currents.yaml', 'test_geometry_15b_output'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/test_geometry_15_screening_currents_1.yaml', 'geometry/output/test_geometry_15_3_output'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/test_geometry_16_screening_currents.yaml', 'test_geometry_16_output'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/test_geometry_17_screening_currents.yaml', 'test_geometry_17_output'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/test_geometry_18_test_SC_1.yaml', 'test_geometry_18_1_output'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/test_geometry_18_test_SC_3.yaml', 'test_geometry_18_3_output'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/test_geometry_/18_test_SC_5.yaml', 'test_geometry_18_5_output'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/test_geometry_18_test_SC_7.yaml', 'test_geometry_18_7_output'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/test_geometry_18_test_SC_9.yaml', 'test_geometry_18_9_output'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/test_geometry_18_test_SC_11.yaml', 'test_geometry_18_11_output'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/test_geometry_19_psi.yaml', 'test_geometry_19_output'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/test_geometry_19_psi_2.yaml', 'test_geometry_19_2_output'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/test_geometry_19_psi_3.yaml', 'test_geometry_19_3_output'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/test_geometry_19_psi_single_coil.yaml', 'test_geometry_19_dp_output'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/test_geometry_21_6D_A1_forces.yaml', 'geometry/output/21_6D_A1_forces'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/test_geometry_21_6D_B5_B6.yaml', 'test_geometry_21_6D_B5_B6_output'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/6D_cooling.yaml', 'geometry/output/test_geometry_6D_cooling_output'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/6D_cooling_B_only.yaml', 'geometry/output/6D_cooling_B_only'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/6D_cooling_B8.yaml', 'geometry/output/6D_cooling_B8'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/6D_cooling_B7.yaml', 'geometry/output/test_geometry_6D_cooling_B7_output'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/6D_cooling_B7_no_map.yaml', 'geometry/output/test_geometry_6D_cooling_B7_no_map'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/6D_cooling_B6.yaml', 'test_geometry_6D_cooling_B6_output'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/6D_cooling_B5.yaml', 'test_geometry_6D_cooling_B5_output'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/6D_cooling_B4.yaml', 'geometry/output/6D_cooling_B4'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/6D_cooling_B3.yaml', 'geometry/output/6D_cooling_B3'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/6D_cooling_B2.yaml', 'geometry/output/6D_cooling_B2'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/6D_cooling_B1.yaml', 'geometry/output/6D_cooling_B1'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/6D_cooling_B7_2_cells.yaml', 'geometry/output/6D_cooling_B7_2_cells'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/6D_cooling_B7_3_cells.yaml', 'geometry/output/6D_cooling_B7_3_cells'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/6D_cooling_B7_3_cells_powered.yaml', 'geometry/output/6D_cooling_B7_3_cells_powered'])

# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/6D_cooling_B1_4_powered.yaml', 'geometry/output/6D_cooling_B1_4_powered'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/6D_cooling_B1_4_non_powered.yaml', 'geometry/output/6D_cooling_B1_4_non_powered'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/6D_cooling_B2_4_powered.yaml', 'geometry/output/6D_cooling_B2_4_powered'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/6D_cooling_B2_4_non_powered.yaml', 'geometry/output/6D_cooling_B2_4_non_powered'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/6D_cooling_B3_4_powered.yaml', 'geometry/output/6D_cooling_B3_4_powered'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/6D_cooling_B3_4_non_powered.yaml', 'geometry/output/6D_cooling_B3_4_non_powered'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/6D_cooling_B4_4_powered.yaml', 'geometry/output/6D_cooling_B4_4_powered'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/6D_cooling_B4_4_non_powered.yaml', 'geometry/output/6D_cooling_B4_4_non_powered'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/6D_cooling_B5_4_powered.yaml', 'geometry/output/6D_cooling_B5_4_powered'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/6D_cooling_B5_4_non_powered.yaml', 'geometry/output/6D_cooling_B5_4_non_powered'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/6D_cooling_B6_4_powered.yaml', 'geometry/output/6D_cooling_B6_4_powered'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/6D_cooling_B6_4_non_powered.yaml', 'geometry/output/6D_cooling_B6_4_non_powered'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/6D_cooling_B7_4_powered.yaml', 'geometry/output/6D_cooling_B7_4_powered'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/6D_cooling_B7_4_non_powered.yaml', 'geometry/output/6D_cooling_B7_4_non_powered'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/6D_cooling_B8_4_powered.yaml', 'geometry/output/6D_cooling_B8_4_powered'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/6D_cooling_B8_4_non_powered.yaml', 'geometry/output/6D_cooling_B8_4_non_powered'])

# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/test_geometry_22_ATLAS_5T_NbTi.yaml', 'test_geometry_22_output'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/test_geometry_23_FCS_1.yaml', 'test_geometry_23_output'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/test_geometry_23_FCS_1.yaml', 'test_geometry_23_thermal_output'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/test_geometry_23_FCS_2.yaml', 'test_geometry_23_2_output'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/test_geometry_23_FCS_3.yaml', 'test_geometry_23_3_output'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/test_geometry_23_FCS_3_5_b.yaml', 'geometry/output/test_geometry_23_FCS_2_output_new'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/test_geometry_23_FCS_3_5_c.yaml', 'test_geometry_23_FCS_c_output'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/test_geometry_23_FCS_3_5_d.yaml', 'test_geometry_23_FCS_d_output'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/test_geometry_23_FCS_3_5_e.yaml', 'test_geometry_23_FCS_e_output'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/test_geometry_23_FCS_3_5.yaml', 'geometry/output/test_geometry_23_FCS_output_new'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'examples/legacy/input_yaml/test_geometry_28_FCS_1517A.yaml', 'geometry/output/test_geometry_28_FCS'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/test_geometry_28_FCS_1517A_1.yaml', 'geometry/output/test_geometry_28_FCS_1'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'examples/legacy/input_yaml/test_geometry_29_AMS_100_v2.yaml', 'geometry/output/test_geometry_29_AMS_100_v2'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'examples/legacy/input_yaml/test_geometry_29_AMS_100_v3.yaml', 'geometry/output/test_geometry_29_AMS_100_v3'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'examples/legacy/input_yaml/test_geometry_30_Patricia.yaml', 'geometry/output/Patricia_Coil'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'examples/legacy/input_yaml/test_geometry_31_Patricia.yaml', 'geometry/output/Patricia_Coil_2'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'examples/legacy/input_yaml/test_geometry_32_Patricia.yaml', 'geometry/output/Patricia_Coil_3'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'examples/legacy/input_yaml/test_geometry_33_6D_B7_NI.yaml', 'geometry/output/6D_B7_NI'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'examples/legacy/input_yaml/test_geometry_34_FCS_750.yaml', 'geometry/output/FCS_750'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'examples/legacy/input_yaml/test_geometry_34_FCS_750.yaml', 'geometry/output/FCS_750_Torch'])
subprocess.call(['python', 'NI_Coil_Geometry.py', 'examples/legacy/input_yaml/test_geometry_34_FCS_750_1.yaml', 'geometry/output/FCS_750_Torch_1'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'examples/legacy/input_yaml/test_geometry_35_FCS_375.yaml', 'geometry/output/FCS_375'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'examples/legacy/input_yaml/test_geometry_35_FCS_375.yaml', 'geometry/output/FCS_375_Torch'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'examples/legacy/input_yaml/test_geometry_35_FCS_375_Paper.yaml', 'geometry/output/FCS_375_paper'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'examples/legacy/input_yaml/test_geometry_36_insulated_gianluca.yaml', 'geometry/output/36_insulated_gianluca'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'examples/legacy/input_yaml/CD_small_coils.yaml', 'geometry/output/CD_small_coils'])

# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/test_geometry_23_FCS_4.yaml', 'test_geometry_23_4_output'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/test_geometryI_23_FCS_5.yaml', 'test_geometry_23_5_output'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/test_geometry_23_FCS_6.yaml', 'test_geometry_23_6_output'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/test_geometry_hans.yaml', 'test_geometry_hans_output'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/test_geometry_24_1.yaml', 'test_geometry_24_1_output'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/40T_solenoid_1.yaml', '40T_solenoid_1'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/40T_solenoid_2.yaml', '40T_solenoid_2'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/40T_solenoid_3.yaml', '40T_solenoid_3'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/FCS_from_EXCEL.yaml', 'geometry/output/FCS_from_EXCEL'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/FCS_from_EXCEL_low_out.yaml', 'geometry/output/FCS_from_EXCEL_low_out'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/tree.yaml', 'tree'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/test_geometry_25_Emma.yaml', 'test_geometry_25_output'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/test_geometry_26_5T_Cryo.yaml', 'test_geometry_26_output'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/test_geometry_27_comp_200.yaml', 'test_geometry_27_2_output'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/test_geometry_27_comp_200_1.yaml', 'geometry/output/test_geometry_27_1_output'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'examples/legacy/input_yaml/test_geometry_27_comp_200_insulated.yaml', 'geometry/output/test_geometry_27_insulated_output'])

## Validations ##
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/valid_4_1_1.yaml', 'valid_4_1_1'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/valid_4_5_1.yaml', 'valid_4_5_1'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/valid_4_10_1.yaml', 'valid_4_10_1'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/valid_4_15_1.yaml', 'valid_4_15_1'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/valid_4_20_1.yaml', 'valid_4_20_1'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/valid_4_25_1.yaml', 'valid_4_25_1'])
#
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/valid_4_1_3.yaml', 'valid_4_1_3'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/valid_4_5_3.yaml', 'valid_4_5_3'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/valid_4_10_3.yaml', 'valid_4_10_3'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/valid_4_15_3.yaml', 'valid_4_15_3'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/valid_4_20_3.yaml', 'valid_4_20_3'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/valid_4_25_3.yaml', 'valid_4_25_3'])
#
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/valid_4_1_5.yaml', 'valid_4_1_5'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/valid_4_5_5.yaml', 'valid_4_5_5'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/valid_4_10_5.yaml', 'valid_4_10_5'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/valid_4_15_5.yaml', 'valid_4_15_5'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/valid_4_20_5.yaml', 'valid_4_20_5'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/valid_4_25_5.yaml', 'valid_4_25_5'])
#
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/valid_4_1_7.yaml', 'valid_4_1_7'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/valid_4_5_7.yaml', 'valid_4_5_7'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/valid_4_10_7.yaml', 'valid_4_10_7'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/valid_4_15_7.yaml', 'valid_4_15_7'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/valid_4_20_7.yaml', 'valid_4_20_7'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/valid_4_25_7.yaml', 'valid_4_25_7'])
#
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/valid_4_1_9.yaml', 'valid_4_1_9'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/valid_4_5_9.yaml', 'valid_4_5_9'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/valid_4_10_9.yaml', 'valid_4_10_9'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/valid_4_15_9.yaml', 'valid_4_15_9'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/valid_4_20_9.yaml', 'valid_4_20_9'])
# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/valid_4_25_9.yaml', 'valid_4_25_9'])

# subprocess.call(['python', 'NI_Coil_Geometry.py', 'geometry/input_yaml/valid_4_1_1.yaml', 'valid_4_1_1'])




import  itertools
import  shutil
import  os
from    pathlib import Path
import  sys
import  logging
import  yaml

class niqcs_para_sweep:
    """
    Generates a parametric sweep over several parameters.
    """

    parameters = ['Current']
    parameter_values = []
    output_values = ['nzpv', 'hot-spot']
    itervalues = []
    input_vars = []
    statics = None

    # Filenames
    pybbq_path = 'pyBBQ.py'
    yaml_param = ''             # Yaml file with the parameters to sweep
    yaml_init = ''              # Yaml file with all input parameters for pyBBQ
    yaml_sweep = ''             # Working yaml file copied from yaml_init
    output_csv = ''
    output_folder = 'Output' + os.sep + 'sweeper'
    result_file = ''
    mode = 'Sweep'  # 'Sweep' is standard, 'Sweep-Full' saves the full output
    mode2 = 'full' # partial or full, partial sweeps over range(len(x)) of all arrays, full makes all combinations
    name = None
    optimize = False
    delimiter = ','

    def __init__(self, sweeper_input_yaml = None, original_yaml=None):
        self.yaml_input = sweeper_input_yaml
        self.yaml_original = original_yaml

        if self.yaml_input is None and self.yaml_original is None:
            self.make_folders()
        else:
            # Loading the vars for the parametric sweep
            with open(self.yaml_input) as file:
                contents = yaml.load(file, Loader=yaml.FullLoader)

            if 'name' in contents.keys():
                setattr(self, 'name', contents['name'])

            if 'sweep' in contents.keys():
                setattr(self, 'sweep', contents['sweep'])

            # Making the output folder
            self.make_folders()

            keys = [i for i in contents['sweep']]
            unpacked_values = []
            for i in keys:
                unpacked_values.append([(i, j) for j in contents['sweep'][i]])

            # Creating the sweeper list
            if self.mode2 == 'full':
                self.itervalues = list(itertools.product(*unpacked_values))
            elif self.mode2 == 'partial':
                self.itervalues = list(zip(*unpacked_values))

            self.start_sweep()

    def uniquify(self):
        # Creates a unique foldername for the simulation
        self.unique_number = 1
        path = self.file_folder / 'sweeper' / 'sweeper_output' / (self.name + "_" + str(self.unique_number))
        while os.path.isdir(path):
            self.unique_number += 1
            path = self.file_folder / 'sweeper' / 'sweeper_output' / (self.name + "_" + str(self.unique_number))
        return path

    def make_folders(self):
        """
            Making the sweeper output folders
        """

        # Creating the sweeper folder
        self.file_folder = Path(os.path.dirname(os.path.basename(__file__)))
        try:
            os.mkdir(self.file_folder / 'sweeper')
        except OSError:
            print("Creation of the directory %s failed" % (self.file_folder / 'sweeper'))
        else:
            print("Successfully created the directory %s " % (self.file_folder / 'sweeper'))

        # Creating the sweeper input folder
        try:
            os.mkdir(self.file_folder / 'sweeper' / 'sweeper_input_yaml')
        except OSError:
            print("Creation of the directory %s failed" % (self.file_folder / 'sweeper'))
        else:
            print("Successfully created the directory %s " % (self.file_folder / 'sweeper'))

        # Creating the sweeper output folder
        try:
            os.mkdir(self.file_folder / 'sweeper' / 'sweeper_output')
        except OSError:
            print("Creation of the directory %s failed" % (self.file_folder / 'sweeper'))
        else:
            print("Successfully created the directory %s " % (self.file_folder / 'sweeper'))

        if self.yaml_input is not None and self.yaml_original is not None and self.name is not None:
            self.output_folder = self.uniquify()

            try:
                os.mkdir(self.output_folder)
            except OSError:
                print("Creation of the directory %s failed" % self.output_folder)
            else:
                print("Successfully created the directory %s " % self.output_folder)

            # Copying the input yaml file.
            self.yaml_original_out = self.output_folder / 'original.yaml'
            try:
                shutil.copy(self.yaml_original, self.yaml_original_out)
                print("File copied successfully.")
            # If source and destination are same
            except shutil.SameFileError:
                print("Source and destination represents the same file.")
            # If there is any permission issue
            except PermissionError:
                print("Permission denied.")
            # For other errors
            except:
                print("Error occurred while copying file.")

            # Copying the input yaml file.
            self.yaml_sweep_out = self.output_folder / 'sweeper_input.yaml'
            try:
                shutil.copy(self.yaml_input, self.yaml_sweep_out)
                print("File copied successfully.")
            # If source and destination are same
            except shutil.SameFileError:
                print("Source and destination represents the same file.")
            # If there is any permission issue
            except PermissionError:
                print("Permission denied.")
            # For other errors
            except:
                print("Error occurred while copying file.")

            # creating a python file to run the sweep:
            with open(self.output_folder / 'run_sweep.py', 'w+') as the_file:
                the_file.write('# !/usr/bin/env python3\n')
                the_file.write('import subprocess\n\n')

    def start_sweep(self):
        """ Starts the parametric sweep """
        print(self.itervalues)
        for i in range(len(self.itervalues)):
            yaml_sweep_n = 'sweep_' + str(i) + '.yaml'
            self.set_state(self.itervalues[i], yaml_sweep_n)
            self.input_vars.append(yaml_sweep_n)


    def set_state(self, change_params, output_yaml):
        """ Changes the parameters in the working yaml file
            Input:  zip of the parameters, for example: [('Current', 10), ('T1', 50)]
        """

        logging.info('Preparing: ' + output_yaml)

        with open(self.yaml_original_out) as f:
            doc = yaml.safe_load(f)

        for i in change_params:
            doc[i[0]] = i[1]

        if 'tag' in doc.keys():
            doc['tag'] = doc['tag'] + '_' + self.name + '_' + str(self.unique_number) + '_' + output_yaml.split('.')[0]

        if self.statics is not None:
            for i in self.statics.keys():
                doc[i] = self.statics[i]

        with open(self.output_folder / output_yaml, 'w') as f:
            yaml.safe_dump(doc, f, default_flow_style=False)

        # creating a python file to run the sweep:
        with open(self.output_folder / 'run_sweep.py', 'a+') as the_file:
            the_file.write('subprocess.call([\'python\', \'NI_Coil_Solver_Torch.py\', \'' + str(self.output_folder / output_yaml) + '\'])\n')

if __name__ == "__main__":
    print(sys.argv)
    if len(sys.argv) == 3:
        print('Loading the provided yaml files: ' + sys.argv[1] + ' and ' + sys.argv[2])
        x = niqcs_para_sweep(sys.argv[1], sys.argv[2])
    else:
        x = niqcs_para_sweep()


#!/usr/bin/env python3

import numpy as np
import matplotlib.pyplot as plt
from 	matplotlib.animation 	import FuncAnimation
from    matplotlib              import cm
import pandas as pd
import yaml
import os
import matplotlib.gridspec as gridspec
from matplotlib import rcParams
from matplotlib.ticker import MultipleLocator

class plotter_v3:
    # This code will make 1 figure with 2 subplots with a result as a function of time: e.g. T(t) or Fr_peak(t).
    # Subplot 1 is generally used for the ramp up.
    # Subplot 2 is generally used for the quench.

    # paths
    input_path_1 = []
    input_path_2 = []
    parameter = None

    # Figures and axes
    fig_square = True
    fig = None
    gs = None
    ax1 = None
    ax2 = None
    plot_every= 1

    max_values = []

    # Data
    input_data = None
    limits = None

    time1 = []
    time2 = []

    max_values1 = []
    max_values2 = []

    maxlim = -1e99
    minlim = 1e99

    def __init__(self, input_1, input_2=None, labels1=None, labels2=None, parameter=None):
        self.input_path_1 = input_1
        self.input_path_2 = input_2
        self.parameter = parameter

        if self.fig_square:
            # Figures and axes
            # self.fig = plt.figure(figsize=(12, 4))
            self.fig = plt.figure(figsize=(12, 3.5))
            self.gs = gridspec.GridSpec(1, 2, width_ratios=[1, 2])
            self.ax1 = self.fig.add_subplot(self.gs[0])
            self.ax2 = self.fig.add_subplot(self.gs[1])
        else:
            # Figures and axes
            self.fig = plt.figure(figsize=(6, 4))
            self.gs = gridspec.GridSpec(1, 2, width_ratios=[1, 1])
            self.ax1 = self.fig.add_subplot(self.gs[0])
            self.ax2 = self.fig.add_subplot(self.gs[1])

        # Making the figures:
        # https://matplotlib.org/stable/gallery/subplots_axes_and_figures/broken_axis.html
        # self.fig, (self.ax1, self.ax2) = plt.subplots(1, 2, sharey=True)
        self.fig.subplots_adjust(wspace=0.035)

        # plot the same data on both axes
        self.load_yaml()

        markers = ['o', 'd', '*', 's', 'v', '^']

        for i in range(len(self.time1)):
            if self.parameter in ['Fr', 'Fz', 'II', 'IR']:
                self.ax1.plot(self.time1[i], np.array(self.max_values1[i])*1e-3, marker=markers[i], mfc='white', label=labels1[i])
            elif self.parameter in ['Fr_density', 'Fz_density']:
                self.ax1.plot(self.time1[i], np.array(self.max_values1[i])*1e-9, marker=markers[i], mfc='white', label=labels1[i])
            else:
                self.ax1.plot(self.time1[i], np.array(self.max_values1[i]), marker=markers[i], mfc='white', label=labels1[i])
            if np.max(self.max_values1[i]) > self.maxlim: self.maxlim = np.max(self.max_values1[i])
            if np.min(self.max_values1[i]) > self.minlim: self.minlim = np.min(self.max_values1[i])

        for i in range(len(self.time2)):
            if self.parameter in ['Fr', 'Fz', 'II', 'IR']:
                self.ax2.plot(np.array(self.time2[i])-self.time2[i][0]+5000.0, np.array(self.max_values2[i])*1e-3, marker=markers[i], mfc='white', label=labels2[i])
            elif self.parameter in ['Fr_density', 'Fz_density']:
                self.ax2.plot(np.array(self.time2[i])-self.time2[i][0]+5000.0, np.array(self.max_values2[i])*1e-9, marker=markers[i], mfc='white', label=labels2[i])
            else:
                self.ax2.plot(np.array(self.time2[i])-self.time2[i][0]+5000.0, np.array(self.max_values2[i]), marker=markers[i], mfc='white', label=labels2[i])
            if np.max(self.max_values2[i]) > self.maxlim: self.maxlim = np.max(self.max_values2[i])
            if np.min(self.max_values2[i]) > self.minlim: self.minlim = np.min(self.max_values2[i])

        # zoom-in / limit the view to different portions of the data
        if self.parameter in ['Fr', 'Fz', 'II', 'IR']:
            self.ax1.set_ylim(0, self.maxlim * 1.1 * 1e-3)
            self.ax2.set_ylim(0, self.maxlim * 1.1 * 1e-3)
        elif self.parameter in ['Fr_density', 'Fz_density']:
            self.ax1.set_ylim(0, self.maxlim * 1.1 * 1e-9)
            self.ax2.set_ylim(0, self.maxlim * 1.1 * 1e-9)
        else:
            self.ax1.set_ylim(0, self.maxlim*1.1)
            self.ax2.set_ylim(0, self.maxlim*1.1)
        # self.ax1.set_xlim(0, 5107.35)  # most of the data

        self.ax1.set_xlim(0, 5000)  # most of the data
        self.ax2.set_xlim(5000, 5002)  # most of the data

        self.ax1.set_xlabel('Time, s')
        self.ax2.set_xlabel('Time, s')

        if self.parameter in ['Fr', 'Fz']:
            self.ax1.set_ylabel('Peak Force, kN')
        elif self.parameter in ['Fr_density', 'Fz_density']:
            self.ax1.set_ylabel('Peak Force Density, N/mm$^3$')
        elif self.parameter in ['I_density']:
            self.ax1.set_ylabel('Peak Current Density, A/mm$^2$')
        elif self.parameter in ['II', 'IR']:
            self.ax1.set_ylabel('Current, kA')
        elif self.parameter == 'T':
            self.ax1.set_ylabel('Temperature, K')
        elif self.parameter in ['Bz', 'Br']:
            self.ax1.set_ylabel('Magnetic Field, T')

        if self.parameter in ['T']:
            self.ax1.legend(loc='upper left')
            self.ax2.legend(loc='upper left')
        else:
            self.ax1.legend(loc='lower right')
            self.ax2.legend(loc='upper right')

        self.ax1.grid()
        self.ax2.grid()


        # hide the spines between ax and ax2
        self.ax1.spines.right.set_visible(False)
        self.ax2.spines.left.set_visible(False)
        self.ax1.xaxis.tick_bottom()
        self.ax1.tick_params(labeltop=False)  # don't put tick labels at the top
        self.ax2.xaxis.tick_bottom()
        self.ax2.tick_params(left = False, labelleft = False)

        # Now, let's turn towards the cut-out slanted lines.
        # We create line objects in axes coordinates, in which (0,0), (0,1),
        # (1,0), and (1,1) are the four corners of the axes.
        # The slanted lines themselves are markers at those locations, such that the
        # lines keep their angle and position, independent of the axes size or scale
        # Finally, we need to disable clipping.

        d = 2.0  # proportion of vertical to horizontal extent of the slanted line
        kwargs = dict(marker=[(-1, -d), (1, d)], markersize=12,
                      linestyle="none", color='k', mec='k', mew=1, clip_on=False)
        self.ax1.plot([1, 1], [0, 1], transform=self.ax1.transAxes, **kwargs)
        self.ax2.plot([0, 0], [0, 1], transform=self.ax2.transAxes, **kwargs)


        plt.setp(self.ax1.get_xticklabels()[-1], visible=False)

        for axis in ['top', 'bottom', 'left']:
            self.ax1.spines[axis].set_linewidth(1.0)

        for axis in ['top', 'bottom', 'right']:
            self.ax2.spines[axis].set_linewidth(1.0)

        self.ax1.xaxis.set_tick_params(direction='in', top='on', width=1.2, length=5)
        self.ax1.yaxis.set_tick_params(direction='in', left='on', width=1.2, length=5)
        self.ax1.xaxis.set_tick_params(direction='in', top='on', which='minor', width=1.0, length=3)
        self.ax1.yaxis.set_tick_params(direction='in', left='on', which='minor', width=1.0, length=3)

        self.ax2.xaxis.set_tick_params(direction='in', top='on', width=1.2, length=5)
        self.ax2.yaxis.set_tick_params(direction='in', right='on', width=1.2, length=5)
        self.ax2.xaxis.set_tick_params(direction='in', top='on', which='minor', width=1.0, length=3)
        self.ax2.yaxis.set_tick_params(direction='in', right='on', which='minor', width=1.0, length=3)



        plt.show()

        # self.load_yaml()

        # self.make_gif()

    def load_yaml(self):
        # Reads the contents of the yaml file
        for i in self.input_path_1:
            with open(i + os.sep + 'Paraview' + os.sep + 'python_files_to_load_' + self.parameter + '.yaml') as file:
                data = yaml.load(file, Loader=yaml.FullLoader)
                self.time1.append(data['time'])
                self.max_values1.append(data['max_value'])

        for i in self.input_path_2:
            with open(i + os.sep + 'Paraview' + os.sep + 'python_files_to_load_' + self.parameter + '.yaml') as file:
                data = yaml.load(file, Loader=yaml.FullLoader)
                self.time2.append(data['time'])
                self.max_values2.append(data['max_value'])

    def make_gif(self):

        self.time_steps = list(self.input_data.keys())
        step_data = self.input_data[self.time_steps[4]]

        # self.limits['minmax'][1] = 0.5 * self.limits['minmax'][1]

        for i in step_data:
            df = pd.read_csv(i)
            self.ax.tricontourf(df['x'].values, df['y'].values, df['v'].values, vmin=self.limits['minmax'][0], vmax=self.limits['minmax'][1], cmap=cm.coolwarm)
            self.ax.tricontourf(-1*df['x'].values, df['y'].values, df['v'].values, vmin=self.limits['minmax'][0], vmax=self.limits['minmax'][1], cmap=cm.coolwarm)

        self.ax.set_aspect('equal', 'box')

        anim = FuncAnimation(self.fig, self.update_gif, frames=np.arange(1, len(self.time_steps), self.plot_every), interval=100)
        # IF YOU GET AN ERROR IN SAVING THE GIF -> DOWNLOAD IMAGEMAGICK
        # anim.save(fname, dpi=120, writer='imagemagick', fps=20)
        fname = self.rpath + os.sep + 'test.gif'
        anim.save(fname, dpi=300, fps=10)
        plt.close('all')

    def update_gif(self, i):
        # Updates the gif
        self.ax.clear()

        print('Frame', i)

        step_data = self.input_data[self.time_steps[i]]

        for j in step_data:
            df = pd.read_csv(j)
            self.ax.tricontourf(df['x'].values, df['y'].values, df['v'].values, vmin=self.limits['minmax'][0], vmax=self.limits['minmax'][1], cmap=cm.coolwarm)
            self.ax.tricontourf(-1*df['x'].values, df['y'].values, df['v'].values, vmin=self.limits['minmax'][0], vmax=self.limits['minmax'][1], cmap=cm.coolwarm)

        self.ax.set_title('t = ' + str(np.round((self.time_steps[i]-self.time_steps[0]) * 1e3)) + ' ms')
        self.ax.set_aspect('equal', 'box')

if __name__ == "__main__":
    # input_1 = ['C:/Tim Mulder/CERNBox/Staff - TE-MPE-PE/Projects/ni-coils/geometry/output/test_geometry_4_MT_output/results/result - 2023-08-29 - 0.1 - 2']
    # # input_2 = ['C:/Tim Mulder/CERNBox/Staff - TE-MPE-PE/Projects/ni-coils/geometry/output/test_geometry_4_MT_output/results/result - 2023-09-08 - 0.1 - 5',
    # #            'C:/Tim Mulder/CERNBox/Staff - TE-MPE-PE/Projects/ni-coils/geometry/output/test_geometry_4_MT_output/results/result - 2023-08-29 - 0.1 - 3']
    # # labels2 = ['CD Full', 'Natural Extr.']
    # input_2 = ['C:/Tim Mulder/CERNBox/Staff - TE-MPE-PE/Projects/ni-coils/geometry/output/test_geometry_4_MT_output/results/result - 2023-09-08 - 0.1 - 5',
    #            'C:/Tim Mulder/CERNBox/Staff - TE-MPE-PE/Projects/ni-coils/geometry/output/test_geometry_4_MT_output/results/result - 2023-09-04 - 0.1 - 2',
    #            'C:/Tim Mulder/CERNBox/Staff - TE-MPE-PE/Projects/ni-coils/geometry/output/test_geometry_4_MT_output/results/result - 2023-08-29 - 0.1 - 6',
    #            'C:/Tim Mulder/CERNBox/Staff - TE-MPE-PE/Projects/ni-coils/geometry/output/test_geometry_4_MT_Cu1_output/results/result - 2023-09-05 - 0.1 - 2',
    #            'C:/Tim Mulder/CERNBox/Staff - TE-MPE-PE/Projects/ni-coils/geometry/output/test_geometry_4_MT_output/results/result - 2023-08-29 - 0.1 - 3',]
    # labels1 = ['Ramp up']
    # labels2 = ['1) CD Full', '2) CD Center', '3) CD Extremities', '4) CD Center + QB', '5) Natural Extr. 0.5']


    input_1 = ['C:/Tim Mulder/CERNBox/Staff - TE-MPE-PE/Projects/ni-coils/geometry/output/test_geometry_23_2_output/results/result - 2023-10-05 - 0.1 - 1',
               'C:/Tim Mulder/CERNBox/Staff - TE-MPE-PE/Projects/ni-coils/geometry/output/test_geometry_23_3_output/results/result - 2023-10-06 - 0.1 - 1',
               'C:/Tim Mulder/CERNBox/Staff - TE-MPE-PE/Projects/ni-coils/geometry/output/test_geometry_23_5_output/results/result - 2023-10-13 - 0.1 - 1',
               'C:/Tim Mulder/CERNBox/Staff - TE-MPE-PE/Projects/ni-coils/geometry/output/test_geometry_23_6_output/results/result - 2023-10-23 - 0.1 - 1',
               'C:/Tim Mulder/CERNBox/Staff - TE-MPE-PE/Projects/ni-coils/geometry/output/test_geometry_23_4_output/results/result - 2023-10-10 - 0.1 - 1',]
    input_2 = ['C:/Tim Mulder/CERNBox/Staff - TE-MPE-PE/Projects/ni-coils/geometry/output/test_geometry_23_2_output/results/result - 2023-10-09 - 0.1 - 1',
               'C:/Tim Mulder/CERNBox/Staff - TE-MPE-PE/Projects/ni-coils/geometry/output/test_geometry_23_3_output/results/result - 2023-10-09 - 0.1 - 1']
    labels1 = ['W/ screening currents', 'W/o screening currents', 'W/ SC 4 mm extr. coils', 'W/ SC 4 mm extr. coils v2',  'W/o SC 4 mm extr. coils']
    labels2 = ['With screening currents', 'Without screening currents']

    # input_1 = ['C:/Tim Mulder/CERNBox/Staff - TE-MPE-PE/Projects/ni-coils/geometry/output/test_geometry_28_FCS/results/result - 2024-05-22 - 0.1 - 1']
    # input_2 = ['C:/Tim Mulder/CERNBox/Staff - TE-MPE-PE/Projects/ni-coils/geometry/output/test_geometry_4_MT_output/results/result - 2023-09-08 - 0.1 - 5',
    #            'C:/Tim Mulder/CERNBox/Staff - TE-MPE-PE/Projects/ni-coils/geometry/output/test_geometry_4_MT_output/results/result - 2023-08-29 - 0.1 - 3']
    # labels2 = ['CD Full', 'Natural Extr.']
    # input_2 = ['C:/Tim Mulder/CERNBox/Staff - TE-MPE-PE/Projects/ni-coils/geometry/output/test_geometry_28_FCS/results/result - 2024-05-28 - 0.1 - 1',
    #            'C:/Tim Mulder/CERNBox/Staff - TE-MPE-PE/Projects/ni-coils/geometry/output/test_geometry_28_FCS/results/result - 2024-05-28 - 0.1 - 2', ]
    # labels1 = ['Ramp up']
    # labels2 = ['1) Slow Protection', '2) Faster Protection']


    ptr = plotter_v3(input_1, input_2, labels1, labels2, 'Fr_density')